/*----------------------------------------------------------------------------
(c) TCTEC Pty Ltd www.tctec.net

lrelayset   (Linux Relay Set)  For Ubuntu , using static libftdi library
Super4 Relay module control application.


10-12-2007	Version 1.0 Created
05-10-2010 Version 1.0 for Ubuntu

 
----------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>
#include <inttypes.h>
#include <string.h>
#include "ftdi.h"

#define VERSION_STRING ("Version 1.0") 
//#define DEBUG	(1) 
#define TRUE (1)
#define FALSE (0) 

void *FunctionLib;		/* handle to shared lib file */
uint32_t (*Function)();		/* Pointer to loaded routine */
const char *dlError;	/* Pointer to error string */

// FTDI constants
//
const uint32_t FT_LIST_NUMBER_ONLY =	0x80000000 ;
const uint32_t FT_LIST_BY_INDEX =		0x40000000 ;
const uint32_t FT_LIST_ALL = 			0x20000000 ;

const uint32_t FT_OPEN_BY_SERIAL_NUMBER =	1 ;
const uint32_t FT_OPEN_BY_DESCRIPTION =		2 ;
const uint32_t FT_OPEN_BY_LOCATION = 		4 ;


const uint32_t FT_OK = 0 ;
const char description[] = "TCTEC USB RELAY" ;




//-------------------------------- Relay functions ----------------------

/*
List connected TCTEC Relays
*/
void listTCTECRelays()
{
	long int numDevices ;
	long int temp ;
	long int Flags ;
	long int id ;
	long int locID ;
	char pcSerialNumber[255] ;
	long int Handle ;	
	long int deviceType ;	
	
	int ret, i;
    struct ftdi_context ftdic;
    struct ftdi_device_list *devlist, *curdev;
    char manufacturer[128], pcDescription[128];    
    struct usb_device *aDevice;    

    if (ftdi_init(&ftdic) < 0)
    {
        fprintf(stderr, "ftdi_init failed\n");
        return;
        
    }

    if ((ret = ftdi_usb_find_all(&ftdic, &devlist, 0x0403, 0x6001)) < 0)
    {
        fprintf(stderr, "ftdi_usb_find_all failed: %d (%s)\n", ret, ftdi_get_error_string(&ftdic));
        return;
    }	
	
	
	
	printf("\n\n\nFTDI Devices = %d\n", ret) ;
	printf("--------------------------------------\n") ;
	
	// get details of connected FTDI devices
	//	
	
	printf("\n\nDevice\t\t\tSerialNumber\n") ;
	printf("---------------------------------------------------------\n") ;
	
	
	
	curdev = devlist;
	
	
	for(temp = 0 ; temp < ret; temp++)
	{
		 aDevice = curdev->dev;  // point to the device 	
		
		 
		 ftdi_usb_get_strings(&ftdic, aDevice,
                              manufacturer, 128,
                              pcDescription, 128,
                              pcSerialNumber, 255);
		
		
			 
		if (strcmp(pcDescription,  description) == 0) 
		{
			// found a relay board
			//
			printf("%s\t\t\t%s\n", pcDescription, pcSerialNumber) ;
		}		
		
		curdev = curdev->next;		
		
	}	
	ftdi_deinit(&ftdic);	
	return;		
	
}

/*
Set the relay state by writing a nibble to the relay board.
This also sets the drive state of the pins to "output"

serialNumber:	The serial number of the relay board
mask:			The bits to set / un-set  (e.g 4 = 0100 = relay 3)
on:				1 = set (on) 0 = unset (off)
*/
int setBits(char *serialNumber, uint8_t mask, int on) 
{
	
	int temp;
	int ret;
	long int Handle ;	
	uint8_t bRelayStates = 0 ;
	uint8_t bitMode ;	
	
	struct ftdi_device_list *devlist, *curdev;
    char manufacturer[128], pcDescription[128];   
    char pcSerialNumber[255] ; 
    struct usb_device *aDevice;
    struct usb_device foundDevice;
    int found = FALSE;
    struct ftdi_context ftdic;
    
	
	
	// open a device and set to outputs, may already be done.
	//
		
	if (ftdi_init(&ftdic) < 0)
    {
        fprintf(stderr, "ftdi_init failed\n");
        
    }
        
    // open the specified device
    //
    if ((ret = ftdi_usb_find_all(&ftdic, &devlist, 0x0403, 0x6001)) < 0)
    {
        fprintf(stderr, "ftdi_usb_find_all failed: %d (%s)\n", ret, ftdi_get_error_string(&ftdic));
        //return EXIT_FAILURE;
    }
     
    curdev = devlist;
        	
	
	for(temp = 0 ; temp < ret; temp++)
	{
		 aDevice = curdev->dev;  // point to the device 			
		 
		 ftdi_usb_get_strings(&ftdic, aDevice,
                              manufacturer, 128,
                              pcDescription, 128,
                              pcSerialNumber, 255);			
		 
		if (strcmp(pcDescription,  description) == 0) 
		{
			// found a relay board
			// is it the one we are looking for ?
			//			
			if (strcmp(pcSerialNumber,  serialNumber) == 0) 
			{
				found = TRUE ;	
				foundDevice = *aDevice;			
			}
			
		}		
		
		curdev = curdev->next;		
		
	}	
    
    if (found == TRUE)
    {
		  		  
		  temp = ftdi_usb_open_dev(&ftdic, &foundDevice);
		 
		  if (temp < 0)
		  {
			  printf("Error opening device -3\n") ;	
			  ftdi_deinit(&ftdic);			  		  
			  return;
		  }
		  
		  ftdi_set_bitmode(&ftdic, 0xFF, BITMODE_BITBANG);
		//  sleep(3);
		 
	      ftdi_read_data(&ftdic, &bRelayStates, 1);
	      
	      
	      bRelayStates = ~bRelayStates ;
	
	      // if turning on, then OR with current state
	      //
	      if (on)
	      {
		    bRelayStates |= mask ;
	      }
	      else
	      {
		    bRelayStates &= ~(mask) ; // turn off
	      }
	      // set the relays, invert before writing out. (0 = on, 1 = off)
	      //
	      bRelayStates = ~bRelayStates ;
		
	      #ifdef DEBUG
	      printf("Relay state = %d\n", (int) bRelayStates) ;
	      #endif			     
	  
	  
	  temp = ftdi_write_data(&ftdic, &bRelayStates, 1);		
	  ftdi_usb_close(&ftdic);
	  }		
	  
	  ftdi_deinit(&ftdic);
	  return(0) ;
}



void printHelpMessage()
{
	printf("%s\n", VERSION_STRING) ;
	printf("\n-------------------------------\n") ;
	printf("TCTEC relay command for UBUNTU\n") ;
	printf("%s\n",VERSION_STRING) ;
	printf("www.tctec.net\n") ;
	printf("\n-------------------------------\n") ;
	printf("Commands:\n") ;
	printf("\n-l\t\tList all connected TCTEC relay modules\n") ;
	printf("\nUSAGE:\n") ;
	printf("lrelayset [-u,-s][SERIAL NUMBER],[MASK][ -tTIME IN MILLISECONDS]\n") ;
	printf("\t\t\t\t-t :optional for momentary action (1 to 10000)\n") ;
	printf("\t\t\t\t-u :{u}nset the relay/s\n") ;
	printf("\t\t\t\t-s :{s}et the relay/s\n") ;
	printf("\t\t\t\tMASK is a number representing bits (01 to 15)\n\n") ;
	printf("-s[SERIAL NUMBER],[MASK]\tSet the relays of a module (Turn ON)\n") ;
	printf("\t\t\t\te.g: -sFTQ57KP1,15\n") ;
	printf("\t\t\t\t( 15 = binary 1111. Therefore all relays on. ) ") ;
	printf("\n\t\t\t\te.g: -uFTQ57KP1,15\n") ;
	printf("\t\t\t\t( 15 = binary 1111. Therefore all relays off. ) ") ;
	printf("\n\n\n") ;		
	
}

int main(int argc, char** argv) 
{		
	char serialNum[255] ="";
	char* serNumEnd ;
	int state ;
	uint16_t momentaryPause = 0 ;
	int on ;  // (1 == on, 0 == off) 
	
		
	if (argc > 1)
	{
		
		// parse arguments
		//
		if (strcmp(argv[1], "-l") == 0)
		{
			listTCTECRelays() ;			
		}
		else if ((strncmp(argv[1], "-s", 2) == 0) || (strncmp(argv[1], "-u", 2) == 0))
		{
						
			#ifdef DEBUG
			printf("Setting relays\n") ;
			#endif
			
			state = -1 ;
			
			if (strncmp(argv[1], "-s", 2) == 0)
			{
				on = 1;
			}
			else
			{
				on = 0 ;
			}
			
			
			// find the serial number in the argument
			//
		
			strncpy(serialNum, (char*)&argv[1][2], 250) ;  // ignore the "-s" or "-u"			
			
			serNumEnd = memchr(serialNum, ',', strlen(serialNum)) ; // find the comma
			if (serNumEnd)
			{				
				// get the state value from the string
				//
				state = atoi(serNumEnd+1) ;
			
				(*serNumEnd) = (char)0 ;  // null terminate the serial number string
			}
			
			#ifdef DEBUG
			printf("SerialNumber = %s\n", serialNum) ;
			printf("State = %d\n", state) ;
			#endif
			
			// validate the state value
			//
			if ((state < 0) || (state > 15))
			{
				printf("Invalid state value.\n") ;
				return(1) ;
			}
			
			// have we specified a momentary action ?
			//			
			if (argc >= 3)
			{
				if (strncmp(argv[2], "-t", 2) == 0)
				{
					momentaryPause = atoi((char*)&argv[2][2]) ;
					
					// must be less than or equal to 10 seconds
					//
					if (momentaryPause > 10000)
					{
						momentaryPause = 0 ;
					}
					
					#ifdef DEBUG
					printf("Momentary Pause selected: %d milli seconds.\n", momentaryPause) ;
					#endif
				}				
			}
			setBits((char*) serialNum, state, on) ;
			
			if (momentaryPause > 0)
			{
				usleep(momentaryPause*1000);
				
				// do the opposite to above, ie if relays on, then turn off
				//
				setBits(serialNum, state, ((~on) & 0x01)) ;
			}						
		}
		else 
		{
			printHelpMessage() ;
		}				
	
	}
	else   // no arguments specified, so print help message
	{
		printHelpMessage() ;
	}	
	exit(0) ;	
	
	return(0) ;
	
}
